#!/usr/bin/env bash

#SHOULD BE 0 IN RELEASE!!!
show_watch=0

sleep_watch=0.3

function watch() {
  if [[ $show_watch -eq 1 ]]; then
    echo "$@"
    [[ $sleep_watch != 0 ]] && sleep $sleep_watch
  fi
}

function expand_gpu_list() {
  for e in `echo $1 | tr ',' ' '`; do
    if [[ $e =~ '-' ]]; then
      eval "for i in {${e/-/..}}; do echo \$i; done"
    else
      echo $e
    fi
  done
}

function miner_ver() {
  local MINER_VER=$BZMINER_VER
  [[ -z $MINER_VER ]] && MINER_VER=$MINER_LATEST_VER
  echo $MINER_VER
}

function miner_config_echo() {
  local MINER_VER=$(miner_ver)
  miner_echo_config_file "/hive/miners/$MINER_NAME/$MINER_VER/config.txt"
}


function bzminer_config_gen_legacy() {
  local MINER_CONFIG="$MINER_DIR/$MINER_VER/config.txt"
  mkfile_from_symlink $MINER_CONFIG

  conf=`cat $MINER_DIR/$MINER_VER/config_global.json | envsubst`

  #merge user config options into main config
  if [[ ! -z $BZMINER_USER_CONFIG ]]; then
    while read -r line; do
      [[ -z $line ]] && continue
      conf=$(jq -s '.[0] * .[1]' <<< "$conf {$line}")
    done <<< "$BZMINER_USER_CONFIG"
  fi

  #merge settings into main config
  [[ -z $BZMINER_ALGO ]] && BZMINER_ALGO="ethash"
  local param='"algorithm": "'$BZMINER_ALGO'"'
  conf=$(jq -s '.[0] * .[1]' <<< "$conf {$param}")

  if [[ -z $BZMINER_TEMPLATE ]]; then
    echo -e "${YELLOW} WALLET is empty${NOCOLOR}"
    return 1
  else
    param='"wallet": "'$BZMINER_TEMPLATE'"'
    conf=$(jq -s '.[0] * .[1]' <<< "$conf {$param}")
  fi

  if [[ -n $BZMINER_WORKER ]]; then
    param='"rig": "'$BZMINER_WORKER'"'
    conf=$(jq -s '.[0] * .[1]' <<< "$conf {$param}")
  fi

  local pools=[]
  #merge pools into main config
  if [[ -z $BZMINER_URL ]]; then
    echo -e "${YELLOW} Pool is empty${NOCOLOR}"
    return 1
  fi
  local protocol=
  local url=

  for t_url in $BZMINER_URL; do
    if [[ $t_url == *://* ]]; then
      if [[ $t_url =~ ([A-Za-z0-9]+)\+([A-Za-z]+)://([^[:space:]]+) ]]; then
        protocol=${BASH_REMATCH[1]}
        url=${BASH_REMATCH[3]}
      else
        echo -e "${YELLOW} Pool address parsing error${NOCOLOR}"
        return 1
      fi
    else
      protocol="stratum"
      url=$t_url
    fi
    [[ $BZMINER_TLS -eq 1 ]] && protocol+="+ssl://" || protocol+="+tcp://"
#  [[ -n $BZMINER_PASS ]] && protocol+="${BZMINER_TEMPLATE}.${BZMINER_WORKER}:${BZMINER_PASS}@"

    pool="${protocol}${url}"
    pools=`jq --null-input --argjson pools "$pools" --arg pool "$pool" '$pools + [$pool]'`
  done

  if [[ -z $pools || $pools == '[]' || $pools == 'null' ]]; then
    echo -e "${RED}No pools configured, using default${NOCOLOR}"
  else
    pools=`jq --null-input --argjson pools "$pools" '{"pool": $pools}'`
    conf=$(jq -s '.[0] * .[1]' <<< "$conf $pools")
  fi

  if [[ -n $BZMINER_PASS ]]; then
    param='"pool_password": "'$BZMINER_PASS'"'
    conf=$(jq -s '.[0] * .[1]' <<< "$conf {$param}")
  fi

  if [[ -n $BZMINER_DISABLE_GPUS ]]; then
    local gpus='[]'
    for gpu in `expand_gpu_list ${BZMINER_DISABLE_GPUS// }`; do #delete spaces in gpuslist
      watch gpu $gpu
      gpu_bus_id=`cat /run/hive/gpu-detect.json | jq -r ".[$gpu].busid" | cut -d '.' -f 1`
      watch gpu_bus_id $gpu_bus_id
      gpu_bus_id_1=`cut -d ':' -f 1 <<< "$gpu_bus_id"`
      watch gpu_bus_id_1 $gpu_bus_id_1
      gpu_bus_id_2=`cut -d ':' -f 2 <<< "$gpu_bus_id"`
      watch gpu_bus_id_2 $gpu_bus_id_2
      (( gpu_bus_num_1=0x$gpu_bus_id_1 ))
      watch gpu_bus_num_1 $gpu_bus_num_1
      (( gpu_bus_num_2=0x$gpu_bus_id_2 ))
      watch gpu_bus_num_2 $gpu_bus_num_2
      gpu_bus_num=${gpu_bus_num_1}:${gpu_bus_num_2}
      watch gpu_bus_num $gpu_bus_num

      param='{"uid": "'$gpu_bus_num'", "start_mining": false}'
      watch param $param

      gpus=`jq --null-input --argjson gpus "$gpus" --argjson param "$param" '$gpus + [$param]'`
      watch "gpus" "$gpus"
      #gpus=$(jq -s '.[0] * .[1]' <<< "$gpus {$param}")
    done

    # conf=$(jq -s '.[0] * .[1]' <<< "$conf {"device_overrides": $gpus}")
    gpus=`jq --null-input --argjson gpus "$gpus" '{"device_overrides": $gpus}'`
    conf=$(jq -s '.[0] * .[1]' <<< "$conf $gpus")
  fi

  echo "$conf" | jq . > $MINER_CONFIG

  watch conf $conf
  [[ $show_watch -eq 1 ]] && sleep 5

  exit 0
}

function bzminer_config_gen_5() {
  local MINER_CONFIG="$MINER_DIR/$MINER_VER/config.txt"
  mkfile_from_symlink $MINER_CONFIG

  conf=`cat $MINER_DIR/$MINER_VER/config_global.json | envsubst`

  #merge user config options into main config
  if [[ ! -z $BZMINER_USER_CONFIG ]]; then
    while read -r line; do
      [[ -z $line ]] && continue
      conf=$(jq -s '.[0] * .[1]' <<< "$conf {$line}")
    done <<< "$BZMINER_USER_CONFIG"
  fi

  #merge params into pool config
  local pool_conf={}

  [[ -z $BZMINER_ALGO ]] && BZMINER_ALGO="ethash"
  local param='"algorithm": "'$BZMINER_ALGO'"'
  pool_conf=$(jq -s '.[0] * .[1]' <<< "$pool_conf {$param}")

  if [[ -z $BZMINER_TEMPLATE ]]; then
    echo -e "${YELLOW} WALLET is empty${NOCOLOR}"
    return 1
  else
    param='"wallet": "'$BZMINER_TEMPLATE'"'
    pool_conf=$(jq -s '.[0] * .[1]' <<< "$pool_conf {$param}")
  fi

  if [[ -n $BZMINER_WORKER ]]; then
    param='"username": "'$BZMINER_WORKER'"'
    pool_conf=$(jq -s '.[0] * .[1]' <<< "$pool_conf {$param}")
  fi

  if [[ -n $BZMINER_PASS ]]; then
    param='"password": "'$BZMINER_PASS'"'
    pool_conf=$(jq -s '.[0] * .[1]' <<< "$pool_conf {$param}")
  fi

  if [[ $BZMINER_LHR -eq 1 ]]; then
    param='"lhr_only": true'
  else
    param='"lhr_only": false'
  fi
  pool_conf=$(jq -s '.[0] * .[1]' <<< "$pool_conf {$param}")

  local pools=[]
  #merge pools into pool config
  if [[ -z $BZMINER_URL ]]; then
    echo -e "${YELLOW} Pool is empty${NOCOLOR}"
    return 1
  fi
  local protocol=
  local url=

  for t_url in $BZMINER_URL; do
    if [[ $t_url == *://* ]]; then
      if [[ $t_url =~ ([A-Za-z0-9]+)\+([A-Za-z]+)://([^[:space:]]+) ]]; then
        protocol=${BASH_REMATCH[1]}
        url=${BASH_REMATCH[3]}
      elif [[ $t_url =~ (http)://([^[:space:]]+) ]]; then
        protocol=${BASH_REMATCH[1]}
        url=${BASH_REMATCH[2]}
      else
        echo -e "${YELLOW} Pool address parsing error${NOCOLOR}"
        return 1
      fi
    else
      protocol="stratum"
      url=$t_url
    fi
    [[ $BZMINER_TLS -eq 1 ]] && protocol+="+ssl://" || protocol+="+tcp://"
#  [[ -n $BZMINER_PASS ]] && protocol+="${BZMINER_TEMPLATE}.${BZMINER_WORKER}:${BZMINER_PASS}@"

    pool="${protocol}${url}"
    pools=`jq --null-input --argjson pools "$pools" --arg pool "$pool" '$pools + [$pool]'`
  done

  pools=`jq --null-input --argjson pools "$pools" '{"url": $pools}'`
  pool_conf=$(jq -s '.[0] * .[1]' <<< "$pool_conf $pools")

  #merge pool config into pool array
  pools_array=`jq --null-input --argjson pool_conf "$pool_conf" '[$pool_conf]'`

  if [[ -n $BZMINER_ALGO2 ]]; then
    #merge second algo params into pool config
    pool_conf={}

    param='"algorithm": "'$BZMINER_ALGO2'"'
    pool_conf=$(jq -s '.[0] * .[1]' <<< "$pool_conf {$param}")

    if [[ -z $BZMINER_TEMPLATE2 ]]; then
      echo -e "${YELLOW} WALLET2 is empty${NOCOLOR}"
      return 1
    else
      param='"wallet": "'$BZMINER_TEMPLATE2'"'
      pool_conf=$(jq -s '.[0] * .[1]' <<< "$pool_conf {$param}")
    fi

    if [[ -n $BZMINER_WORKER2 ]]; then
      param='"username": "'$BZMINER_WORKER2'"'
      pool_conf=$(jq -s '.[0] * .[1]' <<< "$pool_conf {$param}")
    fi

    if [[ -n $BZMINER_PASS2 ]]; then
      param='"password": "'$BZMINER_PASS2'"'
      pool_conf=$(jq -s '.[0] * .[1]' <<< "$pool_conf {$param}")
    fi

    if [[ $BZMINER_LHR2 -eq 1 ]]; then
      param='"lhr_only": true'
    else
      param='"lhr_only": false'
    fi
    pool_conf=$(jq -s '.[0] * .[1]' <<< "$pool_conf {$param}")

    pools=[]
    #merge second algo pools into pool config
    if [[ -z $BZMINER_URL2 ]]; then
      echo -e "${YELLOW} Pool is empty${NOCOLOR}"
      return 1
    fi
    protocol=
    url=

    for t_url in $BZMINER_URL2; do
      if [[ $t_url == *://* ]]; then
        if [[ $t_url =~ ([A-Za-z0-9]+)\+([A-Za-z]+)://([^[:space:]]+) ]]; then
          protocol=${BASH_REMATCH[1]}
          url=${BASH_REMATCH[3]}
        else
          echo -e "${YELLOW} Pool address parsing error${NOCOLOR}"
          return 1
        fi
      else
        protocol="stratum"
        url=$t_url
      fi
      [[ $BZMINER_TLS2 -eq 1 ]] && protocol+="+ssl://" || protocol+="+tcp://"

      pool="${protocol}${url}"
      pools=`jq --null-input --argjson pools "$pools" --arg pool "$pool" '$pools + [$pool]'`
    done

    pools=`jq --null-input --argjson pools "$pools" '{"url": $pools}'`
    pool_conf=$(jq -s '.[0] * .[1]' <<< "$pool_conf $pools")

    #merge second algo pool config into pool array
    pools_array=`jq --null-input --argjson pools_array "$pools_array" --argjson pool_conf "$pool_conf" '$pools_array + [$pool_conf]'`
  fi

  #merge pools array into main config
  pools_array=`jq --null-input --argjson pools_array "$pools_array" '{"pool_configs": $pools_array}'`
  conf=$(jq -s '.[0] * .[1]' <<< "$conf $pools_array")

  if [[ -n $BZMINER_ALGO2 ]]; then
    param='"pool": [0,1]'
    conf=$(jq -s '.[0] * .[1]' <<< "$conf {$param}")
  else
    param='"pool": [0]'
    conf=$(jq -s '.[0] * .[1]' <<< "$conf {$param}")
  fi

  local cpu_indexes_array=
  cpu_indexes_array=`cat $GPU_DETECT_JSON | jq -c '[ . | to_entries[] | select(.value.brand == "cpu") | .key ]'` #'
  local gpu_detect=
  [[ $cpu_indexes_array != '[]' ]] && #remove Internal Gpus
    gpu_detect=$(jq -c "del(.$cpu_indexes_array)" /run/hive/gpu-detect.json) || #"
    gpu_detect=$(cat /run/hive/gpu-detect.json)


  if [[ -n $BZMINER_DISABLE_GPUS ]]; then
    local gpus='[]'
    for gpu in `expand_gpu_list ${BZMINER_DISABLE_GPUS// }`; do #delete spaces in gpuslist
      watch gpu $gpu
      gpu_bus_id=`echo $gpu_detect | jq -r ".[$gpu].busid" | cut -d '.' -f 1`
      watch gpu_bus_id $gpu_bus_id
      gpu_bus_id_1=`cut -d ':' -f 1 <<< "$gpu_bus_id"`
      watch gpu_bus_id_1 $gpu_bus_id_1
      gpu_bus_id_2=`cut -d ':' -f 2 <<< "$gpu_bus_id"`
      watch gpu_bus_id_2 $gpu_bus_id_2
      (( gpu_bus_num_1=0x$gpu_bus_id_1 ))
      watch gpu_bus_num_1 $gpu_bus_num_1
      (( gpu_bus_num_2=0x$gpu_bus_id_2 ))
      watch gpu_bus_num_2 $gpu_bus_num_2
      gpu_bus_num=${gpu_bus_num_1}:${gpu_bus_num_2}
      watch gpu_bus_num $gpu_bus_num

      param='{"uid": "'$gpu_bus_num'", "start_mining": false}'
      watch param $param

      gpus=`jq --null-input --argjson gpus "$gpus" --argjson param "$param" '$gpus + [$param]'`
      watch "gpus" "$gpus"
      #gpus=$(jq -s '.[0] * .[1]' <<< "$gpus {$param}")
    done

    # conf=$(jq -s '.[0] * .[1]' <<< "$conf {"device_overrides": $gpus}")
    gpus=`jq --null-input --argjson gpus "$gpus" '{"device_overrides": $gpus}'`
    conf=$(jq -s '.[0] * .[1]' <<< "$conf $gpus")
  fi

  echo "$conf" | jq . > $MINER_CONFIG

  watch conf $conf
  [[ $show_watch -eq 1 ]] && sleep 5

  exit 0
}

function miner_config_gen() {
  dpkg --compare-versions `miner_ver` "lt" "5.0"
  if [ $? -eq "0" ]; then
    watch bzminer_config_gen_legacy
    bzminer_config_gen_legacy
  else
    watch bzminer_config_gen_5
    bzminer_config_gen_5
  fi
}
